@@ -53,12 +53,19 @@ class Sun:
         solardistance = 1.00014 - 0.01671 * tmp - 0.00014 * temptwo
         return solardistance * 92955807.3  # miles
 
-    def declination(self, l, b):
-        e = (math.pi / 180) * 23.4397
-        return math.asin(math.sin(b) * math.cos(e) + math.cos(b) * math.sin(e) * math.sin(l))
-
+    def solar_declination(self):
+        # Get the current date (in UTC)
+        now = datetime.datetime.utcnow()
+        day_of_year = now.timetuple().tm_yday
+        solar_declination = -23.45 * math.cos(math.radians((360 / 365) * (day_of_year + 10)))
+        return solar_declination
 
 
+    def observerAngle(self, height):
+        return -2.076 * math.sqrt(height) / 60
+    
+    
+    
     def eclipticlongitude(self, anomaly):
         tmp = math.pi / 180 * (
         1.9148 * math.sin(anomaly) + 0.02 * math.sin(2 * anomaly) + 0.0003 * math.sin(3 * anomaly))
@@ -97,35 +104,50 @@ class Sun:
         c = 1.9148*math.sin(m) + 0.02 * math.sin(2 * m) + 0.0003*math.sin(3*m)
         return c
     
-   def calculate_sunrise(self, latitude, longitude, date):
-        """
-        Calculates the time of sunrise for a given latitude, longitude, and date.
+    def moon_alt_az(self, lat, lon, date):
+        # Convert latitude and longitude to radians
+        lat = math.radians(lat)
+        lon = math.radians(lon)
 
-        Arguments:
-        latitude -- the latitude of the location (in decimal degrees)
-        longitude -- the longitude of the location (in decimal degrees)
-        date -- the date for which to calculate sunrise (as a datetime object)
+        # Calculate the Julian date
+        J2000 = 2451545
+        J = date.timestamp() / 86400 + 2440587.5 - J2000
 
-        Returns:
-        A datetime object representing the time of sunrise on the given date at the given location.
-        """
+        # Calculate the moon's position in radians
+        N = math.radians((125.1228 - 0.0529538083 * J) % 360)
+        i = math.radians(5.1454)
+        w = math.radians((318.0634 + 0.1643573223 * J) % 360)
+        a = 60.2666
+        e = 0.054900
+        M = math.radians((115.3654 + 13.0649929509 * J) % 360)
+        E = M + e * math.sin(M) * (1.0 + e * math.cos(M))
+        xv = a * (math.cos(E) - e)
+        yv = a * (math.sqrt(1.0 - e * e) * math.sin(E))
+        v = math.atan2(yv, xv)
+        r = math.sqrt(xv * xv + yv * yv)
+        xh = r * (math.cos(N) * math.cos(v + w) - math.sin(N) * math.sin(v + w) * math.cos(i))
+        yh = r * (math.sin(N) * math.cos(v + w) + math.cos(N) * math.sin(v + w) * math.cos(i))
+        zh = r * (math.sin(v + w) * math.sin(i))
 
-        # Calculate the number of days since January 1st of the current year
-        N = date.timetuple().tm_yday
+        # Calculate the Greenwich sidereal time
+        JD = date.timestamp() / 86400 + 2440587.5
+        T = (JD - 2451545.0) / 36525
+        L0 = math.radians(280.4665 + 36000.7698 * T)
+        dL = math.radians(218.3165 + 481267.8813 * T)
+        GMST0 = L0 + dL
+        SIDTIME = GMST0 + lon
+
+        # Calculate the moon's altitude and azimuth
+        HA = SIDTIME - math.degrees(math.atan2(yh, xh))
+        alt = math.asin(zh / r)
+        az = math.degrees(
+            math.atan2(math.sin(HA), math.cos(HA) * math.sin(lat) - math.tan(math.asin(zh / r)) * math.cos(lat)))
+        if az < 0:
+            az += 360
+
+        return alt, az
 
-        # Calculate the local apparent solar time
-        time_offset = (longitude / 15.0)
-        solar_time = datetime.datetime.combine(date, datetime.time(12)) - datetime.timedelta(hours=time_offset)
 
-        # Calculate the hour angle at sunrise
-        hour_angle = math.degrees(math.acos((math.sin(math.radians(-0.83)) - math.sin(math.radians(latitude)) * math.sin(math.radians(23.44))) / (math.cos(math.radians(latitude)) * math.cos(math.radians(23.44))))
-
-        # Calculate the time of sunrise
-        sunrise = solar_time + datetime.timedelta(minutes=hour_angle * 4)
-
-        return sunrise
 
 
 
-        
-        
